//	EOS_List.h
//  this file handles a EOS_List


//--------------------------- inclusion guards ----------------------
#ifndef _EOS_List_h_
#define	_EOS_List_h_


//--------------------------- include section -----------------------
#include "types.h"
#include "EOS_Macros.h"
#include <stdio.h>
#include <stdlib.h>


//--------------------------- class definition ----------------------
//! this template class represents a list of given objects
template <class T_DATA>
class CEOS_List
{
//--------------------------- construction --------------------------
public:
    // constructor --------------------------------------------------
    //! \internal
    //! the member variables will not be initialized. The init values
    //! have to be all zeros, so the zero init can do it
    CONSTRUCTOR_EMPTY_NO_COPY_AND_ASSIGNMENT(CEOS_List);


//--------------------------- protected types -----------------------
protected:
    // SData --------------------------------------------------------
    //! structure holding the data and the pointer to the next
    //! element
    typedef struct _SData
    {
        _SData* pNext;
        T_DATA xData;
    } SData;


//--------------------------- public types --------------------------
public:
    // TPosition ----------------------------------------------------
    //! data type to be used when storing the position in the list
    //! \code
    //! MyList::TPosition pPosition = MyList.first();
    //! while (pPosition != NULL)
    //! {
    //!     STaskInfo sTaskInfo = MyList.get(pPosition);
    //!     // do something with the data
    //!     pPosition = MyList.next(pPosition);
    //! }
    //! \endcode
    typedef SData* TPosition;


//--------------------------- public access -------------------------
public:
    // add ----------------------------------------------------------
    //! adds an item to the list
    //! \code
    //! CEOS_List<int> MyList;
    //! MyList.add(1);
    //! MyList.add(2);
    //! \endcode
    TPosition add(const T_DATA& xData)
    {
        SData* pNewData = (SData*)malloc(sizeof(SData));
        pNewData->pNext = NULL;
        pNewData->xData = xData;

        if (m_pFirst == NULL)
        {
            m_pFirst = pNewData;
        }
        else
        {
            SData* pLast = m_pFirst;
            while (pLast->pNext != NULL)
            {
                pLast = pLast->pNext;
            }
            pLast->pNext = pNewData;
        }
        return pNewData;
    }

    // insertBefore -------------------------------------------------
    //! inserts an item before the given item
    //! \code
    //! CEOS_List<int> MyList;
    //! CEOS_List<int>::TPosition pPosition = MyList.add(2);
    //! MyList.insertBefore(pPosition,1);
    //! \endcode
    TPosition insertBefore(TPosition pPosition,
                           const T_DATA& xData)
    {
        SData* pNewData = (SData*)malloc(sizeof(SData));
        pNewData->pNext = NULL;
        pNewData->xData = xData;

        if (m_pFirst == pPosition)
        {
            pNewData->pNext = m_pFirst;
            m_pFirst = pNewData;
        }
        else
        {
            SData* pLast = m_pFirst;
            while (pLast->pNext != pPosition)
            {
                pLast = pLast->pNext;
            }
            pNewData->pNext = pLast->pNext;
            pLast->pNext = pNewData;
        }
        return pNewData;
    }

    // insertAfter --------------------------------------------------
    //! inserts an item before the given item
    //! \code
    //! CEOS_List<int> MyList;
    //! CEOS_List<int>::TPosition pPosition = MyList.add(1);
    //! MyList.add(3);
    //! MyList.insertAfter(pPosition,2);
    //! \endcode
    TPosition insertAfter(TPosition pPosition,
                          const T_DATA& xData)
    {
        SData* pNewData = (SData*)malloc(sizeof(SData));
        pNewData->pNext = NULL;
        pNewData->xData = xData;

        SData* pLast = m_pFirst;
        while (pLast != pPosition)
        {
            pLast = pLast->pNext;
        }
        pNewData->pNext = pLast->pNext;
        pLast->pNext = pNewData;
        return pNewData;
    }

    // remove -------------------------------------------------------
    //! removes an item before the given item
    //! \code
    //! CEOS_List<int> MyList;
    //! CEOS_List<int>::TPosition pPosition = MyList.add(1);
    //! MyList.add(2);
    //! MyList.remove(pPosition);   // Removes 1
    //! \endcode
    void remove(TPosition pPosition)
    {
        if (m_pFirst == pPosition)
        {
            m_pFirst = m_pFirst->pNext;
            free(pPosition);
        }
        else
        {
            SData* pLast = m_pFirst;
            while (pLast->pNext != pPosition)
            {
                pLast = pLast->pNext;
            }
            pLast->pNext = pPosition->pNext;
            free(pPosition);
        }
    }

    // first --------------------------------------------------------
    //! \return position of the first element\n
    //! NULL, if the list is empty
    //! \code
    //! MyList::TPosition pPosition = MyList.first();
    //! while (pPosition != NULL)
    //! {
    //!     STaskInfo sTaskInfo = MyList.get(pPosition);
    //!     // do something with the data
    //!     pPosition = MyList.next(pPosition);
    //! }
    //! \endcode
    inline TPosition first()
    {
        return m_pFirst;
    }

    // next ---------------------------------------------------------
    //! \return position of the next element (referenced by the
    //! parameter)\n
    //! NULL, if there is no next element
    //! \code
    //! MyList::TPosition pPosition = MyList.first();
    //! while (pPosition != NULL)
    //! {
    //!     STaskInfo sTaskInfo = MyList.get(pPosition);
    //!     // do something with the data
    //!     pPosition = MyList.next(pPosition);
    //! }
    //! \endcode
    inline static TPosition next(TPosition pPosition)
    {
        return pPosition->pNext;
    }

    // get ----------------------------------------------------------
    //! \return reference to the requested element in the list\n
    //! (referenced by the parameter)\n
    //! \code
    //! MyList::TPosition pPosition = MyList.first();
    //! while (pPosition != NULL)
    //! {
    //!     STaskInfo sTaskInfo = MyList.get(pPosition);
    //!     // do something with the data
    //!     pPosition = MyList.next(pPosition);
    //! }
    //! \endcode
    inline T_DATA& get(TPosition pPosition) const
    {
        return pPosition->xData;
    }


//--------------------------- protected data ------------------------
protected:
    // m_pFirst -----------------------------------------------------
    //! \internal
    //! first element in the list
    SData* m_pFirst;
};


#endif  // Inclusion guard

