//!	EOS_PeriodicTask.h


//--------------------------- inclusion guards ----------------------
#ifndef _EOS_PeriodicTask_h_
#define _EOS_PeriodicTask_h_


//--------------------------- include section -----------------------
#include "EOS_DelayedTask.h"


//--------------------------- class definition ----------------------
//! A CEOS_DelayedTask represents a task, which waits for its
//! execution and winds up its timer again to perform an periodic
//! execution of itself.
//! This class is usually only used by the EOS_Scheduler
//! resp. EOS_RealtimeScheduler
class CEOS_PeriodicTask :
        public CEOS_DelayedTask
{
//--------------------------- construction --------------------------
public:
    // constructor --------------------------------------------------
    //! \internal
    //! Storing the period for winding up the timer when elapsed
    //! the remaining paramters are handed over to the base class
    //! \param pFunc is the C-function
    //! \param nPeriod is the period used to wind up the timer when
    //! elapsed
    //! \param nDelayedExecution is the delay for the first execution
    // constructor --------------------------------------------------
    CEOS_PeriodicTask(void(*pFunc)(),
                      unsigned int nPeriod,
                      unsigned int nDelayedExecution) :
        CEOS_DelayedTask(pFunc,
                         (nDelayedExecution == 0) ? nPeriod : nDelayedExecution),
        m_nPeriod(nPeriod)
    {
        // Nothing to do
    }

    // constructor --------------------------------------------------
    //! \internal
    //! Storing the period for winding up the timer when elapsed
    //! the remaining paramters are handed over to the base class
    //! \param pObject is the pointer to the object holding the
    //! member function being executed
    //! \param pFunc is the member function
    //! \param nPeriod is the period used to wind up the timer when
    //! elapsed
    //! \param nDelayedExecution is the delay for the first execution
    // constructor --------------------------------------------------
    template <class T_OBJECT>
    CEOS_PeriodicTask(T_OBJECT* pObject,
                      void(T_OBJECT::* pFunc)(),
                      unsigned int nPeriod,
                      unsigned int nDelayedExecution) :
        CEOS_DelayedTask(pObject,
                         pFunc,
                         (nDelayedExecution == 0) ? nPeriod : nDelayedExecution),
        m_nPeriod(nPeriod)
    {
        // Nothing to do
    }

    NO_COPY_AND_ASSIGNMENT(CEOS_PeriodicTask);


//--------------------------- public virtual access -----------------
public:
    // timeout ------------------------------------------------------
    //! \internal
    //! virtual function, which winds up the timer with the period
    //! and executes the attached function
    inline virtual void timeout()
    {
        windUp(m_nPeriod);
        execute();
    }

//--------------------------- protected data ------------------------
protected:
    // m_nPeriod ----------------------------------------------------
    //! winds up the timer with this period value
    unsigned int m_nPeriod;
};

#endif
